/* tr.h. Encabezados para el simulador */
#include <stdio.h>
#include "nombres.h"

#define CERCA_CERO 0
#define CERCA_PEOR 1

/* Tipos nuevos */

typedef unsigned short crit;
typedef unsigned int nat;
enum bool {FALSE,TRUE};
enum clase {PER, APER};
enum estados {LISTO, BLOQUEADO, A_COM, DEMORADO};
enum prio {LOW, HIGH};
typedef enum bool boolean;

struct bcp {
   char nombre[MAX_NAME_LENGTH];
   nat Ti, Ci, meta;
   int laxity;
   float criticidad;            /* Un numero real entre 0 y 1 */
   boolean en_conj_crit;
   int tpo_restante;          /* Tiempo que falta p/terminar Ci */
   enum clase tipo;              /* Periodica/aperiodica */
   enum estados estado;
   boolean perdi_meta;
   nat comienzo;     /* Hora de comienzo */
   boolean en_crit; 		/* Dice si esta' en el conj. cri'tico */
} BCP[200], AUX[200], AUX2[200];

struct {
    char alg[MAX_NOM_ALG];       /* Algoritmo usado */
    nat remocion, c_contexto;
    float uso_cpu;              /* % Ci/Ti */
    float Un;                   /* U(n) */
    float sha;                  /* Limite de Sha */
    nat nr_metas, nr_per, nr_aper;     /* Numero total de metas */
    nat metas_perd, metas_per, metas_aper;
    nat g_r, g_r_per, g_r_aper; /* Guarantee Ratios */
    float g_r_pond, g_r_per_pond, g_r_aper_pond;
    float max_gr_pond, max_gr_pond_per, max_gr_pond_aper;
    nat idle_time;
    boolean overload;		/* Hay overload en algun momento? */
} metricas;


struct {
    char alg[MAX_NOM_ALG];       /* Algoritmo usado */
    nat min_rem, min_cc, min_idle;
    nat max_rem, max_cc, max_idle;
    float avg_rem,std_rem,
	  avg_cc, std_cc,
	  avg_uso, std_uso, min_uso, max_uso,
	  avg_gr, std_gr, min_gr, max_gr,
	  avg_gr_per, std_gr_per, min_gr_per, max_gr_per,
	  avg_gr_aper, std_gr_aper, min_gr_aper, max_gr_aper,
	  avg_gr_pond, std_gr_pond, min_gr_pond, max_gr_pond,
	  avg_gr_per_pond, std_gr_per_pond, min_gr_per_pond, max_gr_per_pond,
	  avg_gr_aper_pond, std_gr_aper_pond, min_gr_aper_pond, max_gr_aper_pond;
} t_res [MAX_ALG];      /* Tabla de resultados */



/* Variables globales */

/* a) Estructuras + var. global */

int buscar_tm(void), planif_teo1(void), cc_tm(void), buscar_edf(void),
    planif_1(void), cc_1(void), buscar_llf(void), buscar_muf(void),
    buscar_mmuf(void), nada(void), calc_crit(void), planif_ci_var(void);

int (*algs_plan[MAX_ALG][3])(void) = {
       buscar_tm, planif_teo1, nada,
       buscar_edf, planif_1, nada,
       buscar_llf, planif_1, nada,
       /* buscar_muf, planif_1, nada, */
       buscar_tm, planif_teo1, calc_crit,
       buscar_edf, planif_1, calc_crit,
       buscar_llf, planif_1, calc_crit,
       /* buscar_muf, planif_1, calc_crit, */
       buscar_tm, planif_ci_var, calc_crit,
       buscar_edf,planif_ci_var, calc_crit,
       buscar_llf, planif_ci_var, calc_crit,
       /* buscar_muf, planif_ci_var, calc_crit */
       buscar_tm, planif_ci_var, calc_crit,
       buscar_edf,planif_ci_var, calc_crit,
       buscar_llf, planif_ci_var, calc_crit
};
       /* scheduler, planificabilidad, conj. crtico */

void (*fin_tipo_bench[CANT_T_BENCH])();


void PA1(), PA2(), PA3(), PA4(), PN1(), PN2(), PN3(), PN4(), AA1(), AA2(),
     AA3(), AA4(), AA5(), AN1(), AN2(), AN3(), AN4(), AN5();

void (*tbl_bench[CANT_T_BENCH])(void) = {
	PA1, PA2, PA3, PA4,
	PN1, PN2, PN3, PN4,
	AA1, AA2, AA3, AA4, AA5,
	AN1, AN2, AN3, AN4, AN5 };

/* b) Archivos */
FILE *infile, *res, *resum;

/* c) Funciones */
int (*buscar_prim_lista)(void);
int (*verificar_planif)(void);
int (*set_conj_crit)(void);
void (*gen_tbl_Hartstone)(void);

void init(char *archi);
int seleccionar_algoritmo(int numero, char *nombre);
int buscar_alg(char *nombre);
void generar_TBCP(char *fname);
void get_test_vector(char *fname);
void lookup(char *target, char *seps);
float calc_uso_cpu(nat n);
float calc_uso_per(nat n);
void verificar_metas(int pos);
void simular_ejec(int pos);
void inicializar_mediciones(void);
void generar_aperiodica(void);
void replanificar(void);
void mostrar_TBCP(void);
void imprimir_resultados(void);
void guardar_resultados(void);
void borrar_esp(int pos);
float calc_sumatoria(int i, int k, int l);
void simulador(void);
void inicializar_res(void);
void inicializar_resto_tabla(void);
nat maximo(void);
void verif_args(int argc, char*argv[], char *alg, char*archi);
nat calc_mcd(int);
void guardar_inf_ap(void);
void rest_inf_ap(void);
void salvar_TBCP(void);
void restaurar_TBCP(void);
void perdi_meta(void);
long calc_pot(float []);
void buscar_crit(void);
void ver_crit(float cota);
void reducir();

/* d) Variables */

/* Conj. de tareas sobrecargado; debo parar con los benchmarks;
estoy ejecutando un algoritmo con Ci variable; estoy ejecutando un
benchmark sobrecargado */

boolean overloaded, paro, ci_variable,bm_ovl;

/* Ultima tarea ejecutada, para ver si hubo cambio de contexto */
int anterior=-1;

/* Hasta cuando se ejecuta la simulacio'n; numero de tareas; hora actual */
nat hasta_cuando, nro_tasks, hora;

/* Cantidad de tareas perio'dicas y aperio'dicas */
int cant_aper = 0, cant_per =0;

/* Sumatoria de las tareas ejecutadas antes que esta */
float suma_previos;
boolean imp_c, imp_r, imp_t;   /* Imprime corridas, resultados, tabla */

boolean que_gama;  /* Dice que' funcio'n Gamma estoy usando: cerca de 0 o alejada */

nat clase; /* clase del benchmark */